
%% setting of the system 
function sysInfo = Ax2Bsinwx_settings()


% full model settings
mu = 1; beta = 1; omega = 6; lambda = 10;  
potential_type = 'Ax2Bsin';    % 'A1B1'
switch potential_type 
    case 'Ax2Bsin'; fnA         = @(x) mu*x/2;  fnB   = @(x) sin(tau*x); 
                    gradA       = @(x) mu*x;    gradB = @(x) omega*cos(omega*x); 
                    potential_V = @(x,y) fnA(x)+ lambda/2* sum((y-fnB(x)).^2);  
end

% information of the true system 
Ax2Bsinwx_parameter  = struct(...
       'mu',           mu,...
       'beta',         beta,...
       'omega',        omega,...
       'sigma_1',      sqrt(2/beta),...
       'sigma_2',      sqrt(2/beta),...
       'lambda',       lambda,...
       'potential_V',  potential_V,...
       'fnA',          fnA,...
       'fnB',          fnB, ...
       'gradA',        gradA,...
       'gradB',        gradB...
     );


Sim_Parameter      = struct(...    % full model simulator parameter; 
     'tN',      1*10^5,...
     'dt',      2*10^-4, ...
     'dx',      5*10^-4,...
     'X_int',   [0.5;0], ...
     'x_plot',  -2:0.01:1.5, ...
     'y_plot',  -1:0.01:2.5, ...
     'window',  1000 ...
    );

sysInfo.potential_type      = potential_type; 
sysInfo.sys_parameter = Ax2Bsinwx_parameter; 
%% reduced model settings
rm_info    = struct(...
    'obsgap',    obsgap,...
    't_max',     0.05,...                       % maximum memory length 
    'dt_memory', Sim_Parameter.dt*obsgap, ... % time step-size of reduced model?  
    'max_tN',    1*10^7, ...                 % # time steps of reduced model 
    'Phi',       Phi, ... 
    'beta',      2/Potential_Parameter.sigma^2 ...
    );


 
