%% setting of the system
function [sysInfo,fmDataInfo]  = Double_well_settings(potential_type,scheme)

% full model settings
dimX = 2;
a=1/4; b =1;  beta = 1;   %V(x,y)=a(x^2-1)^2+by^2
%potential_type = 'DoubleWell';    % 'A1B1'
switch potential_type
    case 'DoubleWell' 
        potential_V = @(x,y) a*(x.^2-1).^2 + b*y.^2;
        gradA       = @(x) 4*a*(x.^2-1).*x;   gradB = @(x) 2*b*x;
        % need to be corrected in high dimension case
        hessA       = @(x)  12*a*x^.2 -4*a;  
        hessB       = 2*b;    
        dVx         = @(x) gradA(x);  % note the .* OK for matrix
        dVy         = @(y) gradB(y);
        % need to be corrected in high dimension case
        dVxx        = @(x) hessA(x);
        dVxy        = 0;
        dVyy        = hessB;
        gradV       = @(X) [dVx(X(1,:)); dVy(X(2,:))];
        hessV       = @(X) [dVxx(X(1,:)), dVxy;  dVxy, dVyy];
        LaplaceGradV= @(X) [24*a*X(1,:); 0];          
end


xgrid = -2:0.1:2; ygrid = -2:0.1:2;
plotPotential_2D(potential_V,xgrid,ygrid);

% information of the true system
sysInfo = struct(...
    'potential_type', potential_type,...
    'potential_V',  potential_V,...
    'dimX',         dimX,...
    'a',          a,...
    'b',          b,...
    'beta',         beta,...
    'sigma_1',      sqrt(2/beta),...
    'sigma_2',      sqrt(2/beta),...
    'gradA',        gradA,...
    'gradB',        gradB,...
    'hessA',        hessA,...
    'hessB',        hessB,...
    'xgrid',        xgrid,...
    'ygrid',        ygrid, ...
    'gradV',        gradV,...
    'hessV',        hessV,...
    'LaplaceGradV', LaplaceGradV, ...
    'diffusion',    diag([sqrt(2/beta),sqrt(2/beta)]) ... 
    );

%% setting of the numerical scheme for the true system==>> full model
tN = 1e6; dt = 2e-3; 
Init_distr = 'fixed'; %'Gaussian';
% scheme   = 'SSBE'; %'Split Backward Euler';
ImpSolver= 'Newton';
Newton_maxIte = 40;
Newton_stopCond = dt^2/10;
betaMu = sprintf('beta%i_tN%iw',beta,tN/10000);
fm_type  = [potential_type,'_',betaMu];
fmDataInfo   = struct(...
    'scheme',  scheme,...
    'ImpSolver', ImpSolver, ...
    'dt',      dt,...
    'tN',      tN,...
    'Newton_maxIte', Newton_maxIte,...
    'Newton_stopCond', Newton_stopCond,...
    'type',    fm_type, ... % string for file names
    'dVx',     dVx,...       % used only for XL's code
    'dVy',     dVy, ...
    'dVxx',    dVxx, ...
    'dVxy',    dVxy, ...
    'dVyy',    dVyy, ...
    'betaLambda', betaMu,...
    'getInitial', @(M) getInitial(dimX,M,Init_distr) ... % randn(2,M);
    );
end



function plotPotential_2D(potential_V,xgrid,ygrid)
% plot the potential function

[X,Y] = meshgrid(xgrid,ygrid);
fnVal = potential_V(X,Y);
surf(X,Y,fnVal); colorbar
end
