%% setting of the system
function [sysInfo,fmDataInfo]  = Entropy_Barrier2_settings(potential_type,scheme)

% full model settings
dimX   = 2;
mu     = 0.5;                  % the smaller mu, the more spread 
lambda = 400;                  % 1/lambda is the small parameter
kappa  = 4;
tau    = 2;
omega  = 2;
beta   = 8;                  % noise coefficient

switch potential_type
    case 'EntropyBarrier2' 
        potential_V    = @(x,y) 1/2 * mu * x.^2 + 1/2 * lambda * ( 1 + kappa * exp( -1/2*kappa^2*x.^2 )).*(y - tau*sin(omega*x) ).^2;

        gradA       = @(x,y) mu.*x+(-1).*(1+exp(1).^((-1/2).*kappa.^2.*x.^2).*kappa).*lambda.* ...
                             omega.*tau.*cos(omega.*x).*(y+(-1).*tau.*sin(omega.*x))+(-1/2).* ...
                             exp(1).^((-1/2).*kappa.^2.*x.^2).*kappa.^3.*lambda.*x.*(y+(-1).* ...
                             tau.*sin(omega.*x)).^2; 
        
        gradB       = @(x,y) (1+exp(1).^((-1/2).*kappa.^2.*x.^2).*kappa).*lambda.*(y+(-1).* ...
                             tau.*sin(omega.*x));
        % need to be corrected in high dimension case
        hessA       = @(x,y)  mu+2.*exp(1).^((-1/2).*kappa.^2.*x.^2).*kappa.^3.*lambda.*omega.* ...
                              tau.*x.*cos(omega.*x).*(y+(-1).*tau.*sin(omega.*x))+(1/2).*kappa.* ...    
                              lambda.*((-1).*exp(1).^((-1/2).*kappa.^2.*x.^2).*kappa.^2+exp(1) ...
                                .^((-1/2).*kappa.^2.*x.^2).*kappa.^4.*x.^2).*(y+(-1).*tau.*sin( ...
                              omega.*x)).^2+(1/2).*(1+exp(1).^((-1/2).*kappa.^2.*x.^2).*kappa).* ...
                              lambda.*(2.*omega.^2.*tau.^2.*cos(omega.*x).^2+2.*omega.^2.*tau.* ...
                              sin(omega.*x).*(y+(-1).*tau.*sin(omega.*x)));
                          
        hessB       = @(x,y)  (1+exp(1).^((-1/2).*kappa.^2.*x.^2).*kappa).*lambda;
        
        hessC       = @(x,y)  (-1).*(1+exp(1).^((-1/2).*kappa.^2.*x.^2).*kappa).*lambda.*omega.* ...
                              tau.*cos(omega.*x)+(-1).*exp(1).^((-1/2).*kappa.^2.*x.^2).* ...
                              kappa.^3.*lambda.*x.*(y+(-1).*tau.*sin(omega.*x));
                          
        dVx         = @(x,y) gradA(x,y);  % note the .* OK for matrix
        dVy         = @(x,y) gradB(x,y);
        % need to be corrected in high dimension case
        dVxx        = @(x,y) hessA(x,y);
        dVxy        = @(x,y) hessC(x,y);
        dVyy        = @(x,y) hessB(x,y);
        gradV       = @(X) [dVx(X(1,:),X(2,:)); dVy(X(1,:),X(2,:))];
        hessV       = @(X) [dVxx(X(1,:),X(2,:) ), dVxy(X(1,:),X(2,:) );  dVxy(X(1,:),X(2,:) ), dVyy(X(1,:),X(2,:) )];
        LaGrad1     = @(x,y) (-1).*exp(1).^((-1/2).*kappa.^2.*x.^2).*kappa.^3.*lambda.*x+(-1).* ...
                             kappa.*lambda.*omega.*tau.*((-1).*exp(1).^((-1/2).*kappa.^2.*x.^2) ...
                             .*kappa.^2+exp(1).^((-1/2).*kappa.^2.*x.^2).*kappa.^4.*x.^2).*cos( ...
                             omega.*x).*(y+(-1).*tau.*sin(omega.*x))+(1/2).*kappa.^3.*lambda.* ...
                             x.*(exp(1).^((-1/2).*kappa.^2.*x.^2).*kappa.^2+(-1).*exp(1).^(( ...
                             -1/2).*kappa.^2.*x.^2).*kappa.^4.*x.^2).*(y+(-1).*tau.*sin(omega.* ...
                             x)).^2+(-1).*(1+exp(1).^((-1/2).*kappa.^2.*x.^2).*kappa).*lambda.* ...
                             omega.*(3.*omega.^2.*tau.^2.*cos(omega.*x).*sin(omega.*x)+(-1).* ...
                             omega.^2.*tau.*cos(omega.*x).*(y+(-1).*tau.*sin(omega.*x)))+2.* ...
                             exp(1).^((-1/2).*kappa.^2.*x.^2).*kappa.^3.*lambda.*omega.*x.*(( ...
                             -1).*omega.*tau.^2.*cos(omega.*x).^2+(-1).*omega.*tau.*sin(omega.* ...
                             x).*(y+(-1).*tau.*sin(omega.*x)))+exp(1).^((-1/2).*kappa.^2.*x.^2) ...
                             .*kappa.^5.*x.*((-2).*lambda.*omega.*tau.*x.*cos(omega.*x).*(y+( ...
                             -1).*tau.*sin(omega.*x))+lambda.*(y+(-1).*tau.*sin(omega.*x)).^2)+ ...
                             (-1/2).*exp(1).^((-1/2).*kappa.^2.*x.^2).*kappa.^3.*((-4).* ...
                             lambda.*omega.*tau.*cos(omega.*x).*(y+(-1).*tau.*sin(omega.*x))+ ...
                             lambda.*x.*(2.*omega.^2.*tau.^2.*cos(omega.*x).^2+2.*omega.^2.* ...
                             tau.*sin(omega.*x).*(y+(-1).*tau.*sin(omega.*x))));
                         
        LaGrad2      = @(x,y) 2.*exp(1).^((-1/2).*kappa.^2.*x.^2).*kappa.^3.*lambda.*omega.* ...
                              tau.*x.*cos(omega.*x)+(1+exp(1).^((-1/2).*kappa.^2.*x.^2).*kappa) ...
                              .*lambda.*omega.^2.*tau.*sin(omega.*x)+kappa.*lambda.*((-1).*exp( ...
                              1).^((-1/2).*kappa.^2.*x.^2).*kappa.^2+exp(1).^((-1/2).*kappa.^2.* ...
                              x.^2).*kappa.^4.*x.^2).*(y+(-1).*tau.*sin(omega.*x));                 
        LaplaceGradV= @(X) [LaGrad1(X(1,:),X(2,:)); LaGrad2(X(1,:),X(2,:))];          
end




xgrid = -2:0.1:2; ygrid = -2:0.1:2;
plotPotential_2D(potential_V,xgrid,ygrid);

% information of the true system
sysInfo = struct(...
    'potential_type', potential_type,...
    'potential_V',  potential_V,...
    'dimX',         dimX,...
    'mu',           mu,...
    'lambda',       lambda,...
    'kappa',        kappa, ...
    'tau',          tau, ...
    'omega',        omega, ...
    'beta',         beta,...
    'sigma_1',      sqrt(2/beta),...
    'sigma_2',      sqrt(2/beta),...
    'gradA',        gradA,...
    'gradB',        gradB,...
    'hessA',        hessA,...
    'hessB',        hessB,...
    'hessC',        hessC,...
    'xgrid',        xgrid,...
    'ygrid',        ygrid, ...
    'gradV',        gradV,...
    'hessV',        hessV,...
    'LaplaceGradV', LaplaceGradV, ...
    'diffusion',    diag([sqrt(2/beta),sqrt(2/beta)]) ... 
    );

%% setting of the numerical scheme for the true system==>> full model
M        = 1; tN = 1e7; dt     = 5*10^-5;
obsgap = 5; Init_distr = 'fixed'; %'Gaussian';
% scheme   = 'SSBE'; %'Split Backward Euler';
ImpSolver= 'Newton';
Newton_maxIte = 40;
Newton_stopCond = dt^2/10;
betaMu = sprintf('beta%iM%i_tN%iw',beta, M,tN/10000);
fm_type  = [potential_type,'_',betaMu];
fmDataInfo   = struct(...
    'scheme',  scheme,...
    'ImpSolver', ImpSolver, ...
    'dt',      dt,...
    'tN',      tN,...
    'Newton_maxIte', Newton_maxIte,...
    'Newton_stopCond', Newton_stopCond,...
    'obsgap',  obsgap,...
    'M',       M,...                    % number of trajectories
    'type',    fm_type, ... % string for file names
    'dVx',     dVx,...       % used only for XL's code
    'dVy',     dVy, ...
    'dVxx',    dVxx, ...
    'dVxy',    dVxy, ...
    'dVyy',    dVyy, ...
    'betaLambda', betaMu,...
    'getInitial', @(M) getInitial(dimX,M,Init_distr) ... % randn(2,M);
    );
end



function plotPotential_2D(potential_V,xgrid,ygrid)
% plot the potential function

[X,Y] = meshgrid(xgrid,ygrid);
fnVal = potential_V(X,Y);
surf(X,Y,fnVal); colorbar
end
