%% setting of the system
function [sysInfo,fmDataInfo]  = GradDissCouple_settings(potential_type,scheme)

% full model settings
dimX = 2;
mu1 = .1; beta = 2; mu2=1;
%potential_type = 'GradDissCouple';    % V(x,y)=exp(mu1/2*x.^2+mu2/2.*y.^2)
switch potential_type
    case 'GradDissCouple' 
        potential_V = @(x,y) exp(mu1/2*x.^2+mu2/2.*y.^2);
        % need to be corrected in high dimension case        
        dVx         = @(x,y) exp(mu1/2.*x.^2+mu2/2.*y.^2).*(mu1*x);  % note the .* OK for matrix
        dVy         = @(x,y) exp(mu1/2.*x.^2+mu2/2.*y.^2).*(mu2*y);
        % need to be corrected in high dimension case
        dVxx        = @(x,y)   exp(mu1/2.*x.^2+mu2/2.*y.^2).*((mu1*x).^2+mu1);
        dVxy        = @(x,y)   exp(mu1/2.*x.^2+mu2/2.*y.^2).*(mu1*x).*(mu2*y);
        dVyy        = @(x,y)   exp(mu1/2.*x.^2+mu2/2.*y.^2).*((mu2*y).^2+mu2);
        % Potential Gradient, Hessian and Laplace Gradient
        gradV       = @(X) [dVx(X(1,:),X(2,:)); dVy(X(1,:),X(2,:))];
        hessV       = @(X) [dVxx(X(1,:),X(2,:)),dVxy(X(1,:),X(2,:)); dVxy(X(1,:),X(2,:)), dVyy(X(1,:),X(2,:))];
        LaplaceGradV= @(X) [exp(mu1/2.*X(1,:).^2+mu2/2.*X(2,:).^2).*((mu1*X(1,:)).^3+3*mu1^2*X(1,:)+(mu2*X(2,:))^2*(mu1*X(1,:))+mu2*mu1*X(1,:)); ...
                            exp(mu1/2.*X(1,:).^2+mu2/2.*X(2,:).^2).*((mu2*X(2,:)).^3+3*mu2^2*X(2,:)+(mu2*X(2,:))*(mu1*X(1,:))^2+mu2*mu1*X(2,:))];          
end


xgrid = -2:0.1:2; ygrid = -2:0.1:2;
plotPotential_2D(potential_V,xgrid,ygrid);

% information of the true system
sysInfo = struct(...
    'potential_type', potential_type,...
    'potential_V',  potential_V,...
    'dimX',         dimX,...
    'mu1',          mu1,...
    'mu2',          mu2,...
    'beta',         beta,...
    'sigma_1',      sqrt(2/beta),...
    'sigma_2',      sqrt(2/beta),...
    'xgrid',        xgrid,...
    'ygrid',        ygrid, ...
    'gradV',        gradV,...
    'hessV',        hessV,...
    'LaplaceGradV', LaplaceGradV, ...
    'diffusion',    diag([sqrt(2/beta),sqrt(2/beta)]) ... 
    );

%% setting of the numerical scheme for the true system==>> full model
tN = 1e6; dt = 2e-3; 
Init_distr = 'fixed'; %'Gaussian';
% scheme   = 'SSBE'; %'Split Backward Euler';
ImpSolver= 'Newton';
Newton_maxIte = 40;
Newton_stopCond = dt^2/10;
betaMu = sprintf('beta%ilambda%i_tN%iw',beta,mu2, tN/10000);
fm_type  = [potential_type,'_',betaMu];
fmDataInfo   = struct(...
    'scheme',  scheme,...
    'ImpSolver', ImpSolver, ...
    'dt',      dt,...
    'tN',      tN,...
    'Newton_maxIte', Newton_maxIte,...
    'Newton_stopCond', Newton_stopCond,...
    'type',    fm_type, ... % string for file names
    'dVx',     dVx,...       % used only for XL's code
    'dVy',     dVy, ...
    'dVxx',    dVxx, ...
    'dVxy',    dVxy, ...
    'dVyy',    dVyy, ...
    'betaLambda', betaMu,...
    'getInitial', @(M) getInitial(dimX,M,Init_distr) ... % randn(2,M);
    );
end



function plotPotential_2D(potential_V,xgrid,ygrid)
% plot the potential function

[X,Y] = meshgrid(xgrid,ygrid);
fnVal = potential_V(X,Y);
surf(X,Y,fnVal); colorbar
end
