%% setting of the system
function [sysInfo,fmDataInfo]  = Langevin_MultiParticle_settings(potential_type,scheme)

% full model settings for a damped Langevin dynamics system in one dimensional space 
dimX = 50; % number of particles, use free boundary conditions
heavy_pair_idx=[24, 25]; % index of heavy mass
% mass matrix
m1=1; ratio=1; m2=ratio*m1;
% generate Mass matrix M
temp=m2*ones(dimX,1); temp(heavy_pair_idx)=m1*ones(size(heavy_pair_idx));
M=diag(temp); 
% potential paremeter; inverse temperature; friction constant
mu = 1; para_k=1; beta = 1; gamma=1; % TO DO: spring and friction constant could be adapted to each individual particle
%potential_type = 'LangLocal';    % H(x,p)=V(x)+K(p)
switch potential_type
    case 'Langevin_MultiParticle' 
        potential_V = @(X) mu/4*para_k*((X(1:dimX-1,:)-X(2:dimX,:)).^2-1).^2;
        kinetic_K   = @(p) 1/2*(p)'*inv_M*p; % as P saved as a column vector
        Hamiltonian_H = @(X,p) potential_V(X)+kinetic_K(p);
        % TO DO: need to be corrected in high dimension case, done. TO DO: handle more general interaction potential        
        dVx         = @(X) mu*para_k*[-((X(2,:)-X(1,:)).^2-1)*(X(2,:)-X(1,:));...
                           ((X(2:dimX-1,:)-X(1:dimX-2,:)).^2-1).*(X(2:dimX-1,:)-X(1:dimX-2,:))-((X(3:dimX,:)-X(2:dimX-1,:)).^2-1).*(X(3:dimX,:)-X(2:dimX-1,:));...
                           ((X(dimX,:)-X(dimX-1,:)).^2-1)*(X(dimX,:)-X(dimX,:)) ]; % as X saved as a column vector
        dKp         = @(p) inv_M*p; % as P saved as a column vector
        
        dVxx        = @(X) zeros(2,2); % TO DO: compute it correctly
        dVxxx       = @(X) 0; % TO DO: compute it correctly        
        dKpp        = 0;
        
       % gradV in fact=-RHS force; RHS = [0, 1; -1, 0]*(grad_H)+[0, 0; 0, -gamma]*[x; p]= Hamiltonian force+friction force
       % In the solver function, it needs negative RHS, i.e. negative drift force
%         gradV       = @(X) -([0, 1; -1, 0]* [dVx(X(1,:)) ; dKp(X(2,:))]+[0, 0;0,-gamma]*[(X(1,:)); (X(2,:))]);
%         hessV       = @(X) -([0, 1; -1, 0]*([dVxx(X(1,:)),0;0, dKpp] )+[0,0;0,-gamma]);
%         LaplaceGradV= @(X) -([0, 1; -1, 0]*([dVxxx(X(1,:)),0;0, 0] ));  


 % TO DO: need to be corrected in high dimension case        
        gradV       = @(X) 0;
        hessV       = @(X) 0;
        LaplaceGradV= @(X) 0;          
        
end


xgrid = -2:0.1:2; ygrid = -2:0.1:2;
%plotHamiltonian_2D(Hamiltonian_H,xgrid,ygrid);

% information of the true system
sysInfo = struct(...
    'potential_type', potential_type,...
    'Hamiltonian_H',  Hamiltonian_H,...
    'M',              M,...
    'dimX',           dimX,...
    'heavy_pair_idx', heavy_pair_idx, ...
    'mu',             mu,...
    'para_k',         para_k,...
    'gamma',          gamma,...
    'beta',           beta,...
    'sigma_1',        0,...
    'sigma_2',        sqrt(2/beta),...
    'xgrid',          xgrid,...
    'ygrid',          ygrid, ...
    'gradV',          gradV,...
    'hessV',          hessV,...
    'LaplaceGradV',   LaplaceGradV, ...
    'diffusion',      diag([0,sqrt(2/beta)]) ... 
    );

%% setting of the numerical scheme for the true system==>> full model
tN = 1e6; dt = 1e-2; 
Init_distr = 'fixedMD'; %'Gaussian';
% scheme   = 'SSBE'; %'Split Backward Euler';
ImpSolver= 'Newton';
Newton_maxIte = 40;
Newton_stopCond = dt/10;
betaMu = sprintf('beta%ilambda%i_tN%iw',beta,mu, tN/10000);
fm_type  = [potential_type,'_',betaMu];
fmDataInfo   = struct(...
    'scheme',  scheme,...
    'ImpSolver', ImpSolver, ...
    'dt',      dt,...
    'tN',      tN,...
    'Newton_maxIte', Newton_maxIte,...
    'Newton_stopCond', Newton_stopCond,...
    'type',    fm_type, ... % string for file names
    'dVx',     dVx,...       % used only for XL's code
    'dVxx',    dVxx, ...
    'dVxxx',    dVxxx, ...
    'betaLambda', betaMu,...
    'getInitial', @(M) getInitial(dimX,M,Init_distr) ... % randn(2,M);
    );
end



function plotHamiltonian_2D(Hamiltonian_H,xgrid,ygrid)
% plot the potential function

[X,Y] = meshgrid(xgrid,ygrid);
fnVal = Hamiltonian_H(X,Y);
surf(X,Y,fnVal); colorbar
end
