%% setting of the system
function [sysInfo,fmDataInfo]  = Lorenz_settings(potential_type,scheme)

% full model settings for a Lorenz dynamics system in three dimensional space 
dimX = 3;
% potential paremeters; inverse temperature beta
mu=10; gamma=28; b=8/3; beta = 1;
%eyeMatrix   = eye(dimX,dimX);
potential_type = 'Lorenz';    
switch potential_type
    case 'Lorenz' 
               
       % gradV in fact=-RHS force; 
       % In the solver function, it needs the negative RHS, i.e. negative drift force
        gradV       = @(X) -([mu*(X(2,:)-X(1,:)); gamma*X(1,:)-X(2,:)-X(1,:).*X(3,:); X(1,:).*X(2,:)-b*X(3,:)]);
        hessV       = @(X) -([-mu, mu,0; gamma-X(3,:), -1, -X(1,:); X(2,:), X(1,:),-b]);
        LaplaceGradV= @(X) [0;0;0];          
end


xgrid = -2:0.1:2; ygrid = -2:0.1:2;

% information of the true system
sysInfo = struct(...
    'potential_type', potential_type,...  
    'dimX',           dimX,...
    'mu',             mu,...
    'gamma',          gamma,...
    'b',              b,...
    'beta',           beta,...
    'sigma_1',        0,...
    'sigma_2',        sqrt(2/beta),...
    'xgrid',          xgrid,...
    'ygrid',          ygrid, ...
    'gradV',          gradV,...
    'hessV',          hessV,...
    'LaplaceGradV',   LaplaceGradV, ...
    'diffusion',      diag([sqrt(2/beta),sqrt(2/beta),0]) ... 
    );

%% setting of the numerical scheme for the true system==>> full model
tN         = 6e6;   dt = 5*1e-4; ratio = 1e-3/dt; 
Init_distr = 'fixed'; %'Gaussian';
% scheme   = 'SSBE'; %'Split Backward Euler'; 
gapseq = ratio*[1, 2, 4, 10, 20, 40, 80, 120, 160,200];%  

ImpSolver       = 'Newton';
Newton_maxIte   = 40;
Newton_stopCond = dt/100;
betaMu = sprintf('beta%ilambda%i_tN%iw',beta,mu,tN/10000);
fm_type  = [potential_type,'_',betaMu];
fmDataInfo   = struct(...
    'scheme',  scheme,...
    'ImpSolver', ImpSolver, ...
    'dt',      dt,...
    'tN',      tN,...
    'gapseq',  gapseq,...
    'Newton_maxIte', Newton_maxIte,...
    'Newton_stopCond', Newton_stopCond,...
    'type',    fm_type, ... % string for file names
    'betaLambda', betaMu,...
    'getInitial', @(M) getInitial(dimX,M,Init_distr) ... % randn(2,M);
    );
end



