function [sysInfo,fmDataInfo] = entropyBarrier(potential_type,scheme)
%  Xingjie's Entopy Barrier with potential
%      V(x,y) = mu/2 * fnA(x) + lambda/2* |y - fnB(x)|^2
%{
Gradient system    dX = -gradV(X)dt + sigma dW; with 
      dx1 = - grad(fnA)(x1) dt + lambda*<x2-fnB(x1),grad(fnB(x1))> dt + sigma_1 dW_1(t)
      dx2 = - lambda[x2- B(x1)]dt + sigma_2 dW_2(t)
Write it in vector form.  
%}
%mu = 0.1; beta = 1; omega = 6; lambda = 10;  
dimX = 2; 
mu   = 1; beta = 1; omega = 3*pi; lambda = 50;  
% potential_type = 'Ax2Bsin';    % 'Ax2Bsin'   Ax4Bsin
switch potential_type 
       case 'Ax2Bsin'       % 
        fnA         = @(x)  mu*x.^2/2;  fnB   = @(x) sin(omega*x);
        gradA       = @(x)  mu*x;       gradB = @(x) omega*cos(omega*x);  
        % need to be corrected in high dimension case
        hessA       = @(x)  mu;         hessB = @(x) -omega.^2.*sin(omega*x);
        potential_V = @(x,y) fnA(x)+ lambda/2* (y-fnB(x)).^2;
        dVx         = @(x,y) gradA(x) - lambda* (y-fnB(x)).*gradB(x);  % note the .* OK for matrix
        dVy         = @(x,y) lambda* (y-fnB(x));
        % need to be corrected in high dimension case
        dVxx        = @(x,y) hessA(x)+lambda*(gradB(x)).*(gradB(x))-lambda*(y-fnB(x)).*hessB(x);
        dVxy        = @(x,y) lambda*(-gradB(x));
        dVyy        = @(x,y) lambda;
        gradV       = @(X) [dVx(X(1,:),X(2,:)); dVy(X(1,:),X(2,:))];
        hessV       = @(X) [dVxx(X(1,:),X(2,:)), dVxy(X(1,:),X(2,:));  dVxy(X(1,:),X(2,:)), dVyy(X(1,:),X(2,:))];
        %LaplaceGradV= @(X) [lambda*omega^3* ( X(2,:).*cos(omega*X(1,:)) - 2*cos(2*omega*X(1,:))); ...
                              %lambda*omega^2*sin(omega*X(1,:))];    
        LaplaceGradV= @(X) [lambda*omega^3* ( X(2,:).*cos(omega*X(1,:)) - 2*sin(2*omega*X(1,:))); ...
                              lambda*omega^2*sin(omega*X(1,:))];                          
                          
    case 'Ax4Bsin'   
        fnA         = @(x)  mu*x.^4/4;  fnB   = @(x) sin(omega*x);
        gradA       = @(x)  mu*(x.^3);       gradB = @(x) omega*cos(omega*x);
        % need to be corrected in high dimension case
        hessA       = @(x)  3*mu*(x.^2);         hessB = @(x) -omega.^2.*sin(omega*x);
        potential_V = @(x,y) fnA(x)+ lambda/2* (y-fnB(x)).^2;
        dVx         = @(x,y) gradA(x) - lambda* (y-fnB(x)).*gradB(x);  % note the .* OK for matrix
        dVy         = @(x,y) lambda* (y-fnB(x));
        % need to be corrected in high dimension case
        dVxx        = @(x,y) hessA(x)+lambda*(gradB(x)).*(gradB(x))-lambda*(y-fnB(x)).*hessB(x);
        dVxy        = @(x,y) lambda*(-gradB(x));
        dVyy        = @(x,y) lambda;
        gradV       = @(X) [dVx(X(1,:),X(2,:)); dVy(X(1,:),X(2,:))];
        hessV       = @(X) [dVxx(X(1,:),X(2,:)), dVxy(X(1,:),X(2,:));  dVxy(X(1,:),X(2,:)), dVyy(X(1,:),X(2,:))];
        %LaplaceGradV= @(X) [6*mu*X(1,:) + lambda*omega^3* ( X(2,:).*cos(omega*X(1,:)) - 2*cos(2*omega*X(1,:))); ...
                             %lambda*omega^2*sin(omega*X(1,:))]; 
        LaplaceGradV= @(X) [6*mu*X(1,:) + lambda*omega^3* ( X(2,:).*cos(omega*X(1,:)) - 2*sin(2*omega*X(1,:))); ...
                             lambda*omega^2*sin(omega*X(1,:))]; 
end

 xgrid = -4:0.1:4; ygrid = -5:0.1:5; 
 plotPotential_2D(potential_V,xgrid,ygrid); 


% information of the true system 
sysInfo = struct(...       
       'potential_type', potential_type,...
       'potential_V',  potential_V,...
       'dimX',         dimX,...
       'mu',           mu,...
       'beta',         beta,...
       'omega',        omega,...
       'sigma_1',      sqrt(2/beta),...
       'sigma_2',      sqrt(2/beta),...
       'lambda',       lambda,...       
       'fnA',          fnA,... 
       'fnB',          fnB, ...
       'gradA',        gradA,...
       'gradB',        gradB,...
       'hessA',        hessA,...
       'hessB',        hessB,...
       'xgrid',        xgrid,...
       'ygrid',        ygrid, ...
       'gradV',        gradV,...
       'hessV',        hessV,...
       'LaplaceGradV', LaplaceGradV, ...
       'diffusion',    diag([sqrt(2/beta),sqrt(2/beta)]) ...
   );

%% setting of the numerical scheme for the true system==>> full model
 tN = 1e6; dt = 1e-4; Init_distr = 'Gaussian'; 
% scheme   = 'SSBE'; %'Split Backward Euler';
ImpSolver= 'Newton';
Newton_maxIte = 40;
Newton_stopCond = dt^2/10;
betaLambda = sprintf('beta%ilambda%i_tN%iw',beta,lambda,tN/10000); 
fm_type  = [potential_type,'_',betaLambda]; 
fmDataInfo   = struct(...
    'scheme',  scheme,...
    'ImpSolver', ImpSolver, ...
    'dt',      dt,... 
    'tN',      tN,...
    'Newton_maxIte', Newton_maxIte,...
    'Newton_stopCond', Newton_stopCond,...
    'type',    fm_type, ... % string for file names
    'dVx',     dVx,...       % used only for XL's code
    'dVy',     dVy, ...      
    'dVxx',    dVxx, ...
    'dVxy',    dVxy, ...
    'dVyy',    dVyy, ...
    'betaLambda', betaLambda,...
    'getInitial', @(M) getInitial(dimX,M,Init_distr) ... % randn(2,M);
     );
end



function plotPotential_2D(potential_V,xgrid,ygrid)
% plot the potential function 

[X,Y] = meshgrid(xgrid,ygrid); 
fnVal = potential_V(X,Y);
surf(X,Y,fnVal); colorbar
end
