function [sysInfo,fmDataInfo] = potential1D_backup(potential_type,scheme)
%  1D gradient equation with potential
%      V(X) = mu/4 * fnA(X), where fnA(x) =x^4
%{
Gradient system    dX = -gradV(X)dt + diff_b(X) dW; with 
Write it in vector form.  
%}
dimX = 1; 
mu   = 1; beta = 1; diff_b = sqrt(2/beta); 
switch potential_type 
    case 'x4'       %
        fnV         = @(x)  mu*x.^4/4;
        gradV       = @(x)  mu*x.^3;
        hessV       = @(x)  mu*3*x.^2;
        potential_V = @(x) fnV(x);
        LaplaceGradV= @(x) 6*mu*x;
end

 xgrid = -4:0.1:4; 
 plotPotential_1D(potential_V,xgrid); 


% information of the true system 
sysInfo = struct(...       
       'potential_type', potential_type,...
       'potential_V',  potential_V,...
       'mu',           mu,...
       'dimX',         dimX,...
       'beta',         beta,...
       'diffusion',    diff_b,...
       'gradV',        gradV,...
       'hessV',        hessV,...
       'LaplaceGradV', LaplaceGradV ...
   );


%% setting of the numerical scheme for the true system==>> full model
M        = 1; tN = 2*1e6; dt = 1e-2; obsgap = 1; Init_distr = 'Gaussian'; 
% scheme   = 'Split Backward Euler';
ImpSolver= 'Newton';
Newton_maxIte   = 40;
Newton_stopCond = dt^2/10;
betamu = sprintf('beta%iMu%iM%i_tN%iw',beta,mu, M,tN/10000); 
fm_type  = [potential_type,'_',betamu]; 
fmDataInfo   = struct(...
    'scheme',  scheme,...
    'ImpSolver', ImpSolver, ...
    'dt',      dt,... 
    'tN',      tN,...
    'Newton_maxIte', Newton_maxIte,...
    'Newton_stopCond', Newton_stopCond,...
    'obsgap',  obsgap,...
    'M',       M,...                    % number of trajectories 
    'type',    fm_type, ... % string for file names
    'betaLambda', betamu,...
    'getInitial', @(M) getInitial(dimX,M,Init_distr) ... % randn(2,M);
     );

end

% It is a function in /sysSolver/ now. 
%{
function init = getInitial(d,M,distr)
% generate samples from initial disribution
switch distr
    case 'Gaussian'; init = randn(d,M);
    case 'Unif';     init = rand(d,M); 
end
end
%}

function plotPotential_1D(potential_V,xgrid)
% plot the potential function 
 
fnVal = potential_V(xgrid);
figure; plot(xgrid,fnVal); xlabel('x'); ylabel('Potential');
end
