function [cvec,res,LSE_Abres] = LSE_Axb(Xn1,termsInUse,showRank,lse_entrywise)
% compute the LSE 
%     coefs = min | Xn1 - coefs*termsInUse |^2 
% e.g. Y(n+1) = Yn - deltat * terms(Yn,deltat)  has Xn1 = (Y(n+1)-Yn)/deltat; 
%     coefs = Amat\bvec;     Amat = <terms,terms>; bvec  = <Xn1,terms>;  
%{
 Input
    termsInUse  - a cell with size = [Npar,1] 
 Output
    coefs      - coefficients
    res        - residual of regression, provides variance
    LSE_Abres  - the LSE matrix and vectors, all 
 By Fei Lu, 2020-12-29
%}

if exist('lse_entrywise','var') && lse_entrywise == 1 
    [Amat,bvec,cvec,res,cvecPinv,resPinv] = LSE_entrywise(termsInUse,Xn1,showRank);
else
    [Amat,bvec,cvec,res,cvecPinv,resPinv] = LSE_nonEntrywise(termsInUse,Xn1,showRank);
end

LSE_Abres.Amat     = Amat;      LSE_Abres.bvec    = bvec;  
LSE_Abres.cvec     = cvec;      LSE_Abres.res     = res;
LSE_Abres.cvecPinv = cvecPinv;  LSE_Abres.resPinv = resPinv; 

end

function [Amat,bvec,cvec,res,cvecPinv,resPinv] = LSE_nonEntrywise(termsInUse,Xn1,showRank)
% compute the LSE non-entrywisely ===.>>> the coefficients are sthe same for all components of the terms;
% This seems reasonable for the derived terms; but not good for the
% residual when multiscale entries of Xn1, because not all components have the same scale of residual
    [Npar,~]   = size(termsInUse);   % number of parameters  termInUse is a cell(Npar,1)
    [dimX, tN] = size(Xn1); 
    Amat       = zeros(Npar,Npar);   bvec = zeros(Npar,1);
    for i=1:Npar
        term_i  = termsInUse{i};   % size = dimX x tN
        bvec(i) = sum(sum(Xn1.*term_i) )/tN;
        for j= 1:i
            term_j    = termsInUse{j};
            Amat(i,j) = sum(sum(term_i.*term_j))/tN;
            Amat(j,i) = Amat(i,j);
        end
    end
    cvec     = Amat\bvec;        cterms = 0;        % solve Ax=b by A\b
    cvecPinv = pinv(Amat)*bvec;  cterms2 = 0;          % solve Ax=b by pseudo-inverse
    for i=1:Npar                                          % compute residual
        cterms  = cterms + cvec(i)*termsInUse{i};
        cterms2 = cterms2 + cvecPinv(i)*termsInUse{i};
    end
    res      = norm(Xn1 - cterms, 'fro')^2/(2*tN);
    resPinv  = norm(Xn1 - cterms2, 'fro')^2/(2*tN);
    
    rankA  = rank(Amat);
    if exist('showRank','var') && showRank == 1
        fprintf('Rank Amat = %i, Npar= %i, nonEntrywise \n',rankA, Npar);
        eigenA = eig(Amat);  fprintf('EigenA: '); disp(eigenA');
    end
   
end

function [Amat,bvec,cvec,res,cvecPinv,resPinv] = LSE_entrywise(termsInUse,Xn1,showRank)
% compute the LSE entrywisely. The coefficients are entrywisely LSE, which
% may not respect the derived terms (which are supposed to be in bundle). 
% It is good for the proposed terms, and takes care of the multiscale residuals. 

[Npar,~]   = size(termsInUse);   % number of parameters  termInUse is a cell(Npar,1)
[dimX, tN] = size(Xn1); 
Amat       = zeros(Npar,Npar,dimX);   bvec    = zeros(Npar,dimX);
cvec       = bvec;                    res     = zeros(1,dimX); 
cvecPinv   = bvec;                    resPinv = zeros(1,dimX); 
for k=1:dimX
    
    for i=1:Npar
        % assemble Amat and bvec
        term_i  = termsInUse{i};               % size = dimX x tN
        bvec(i,k) = sum( Xn1(k,:).*term_i(k,:) ) /tN;
        for j= 1:i                             
            term_j      = termsInUse{j};
            Amat(i,j,k) = sum(term_i(k,:).*term_j(k,:) ) /tN;
            Amat(j,i,k) = Amat(i,j,k);
        end
    end
    Amatk     = Amat(:,:,k);         btemp = bvec(:,k); 
    cveck     = Amatk\btemp;         cterms1 = 0;          % solve Ax=b by A\b
    cvecPinvk = pinv(Amatk)*btemp;   cterms2 = 0;          % solve Ax=b by pseudo-inverse
    for i=1:Npar                                   % compute residual
        term_i  = termsInUse{i}; 
        cterms1 = cterms1 + cveck(i)*term_i(k,:);
        cterms2 = cterms2 + cvecPinvk(i)*term_i(k,:);
    end
    res(k)      = norm(Xn1(k,:) - cterms1, 'fro')^2/(2*tN);
    resPinv(k)  = norm(Xn1(k,:) - cterms2, 'fro')^2/(2*tN);    
    cvec(:,k)   = cveck;      cvecPinv(:,k) = cvecPinvk; 
    
    rankA     = rank(Amatk);
    if exist('showRank','var') && showRank == 1
        fprintf('Rank Amat = %i, Npar= %i , entry x%i\n',rankA, Npar,k);
        eigenA = eig(Amatk);  fprintf('EigenA: '); disp(eigenA');
    end
end
end
