function [cvec,res,LSE_Abres] = LSE_Axb_MatHess(Xn1,termsInUse,MatHessV,printON)
% compute the LSE 
%     coefs = min | Xn1 - coefs*termsInUse |_Hess ^2 
%     Xn1 = coefs*terms + Hess*eta     % e.g. Xn1 = (Y(n+1)-Yn)/deltat; 
%     coefs = Amat\bvec;     Amat = <terms,terms>; bvec  = <Xn1,terms>;  
%{
 Input
    termsInUse  - a cell with size = [Npar,1] 
 Output
    coefs      - coefficients
    res        - residual of regression, provides variance
    LSE_Abres  - the LSE matrix and vectors, all 
 By Fei Lu, 2020-12-29
%}

[Npar,~] = size(termsInUse);   % number of parameters
Amat = zeros(Npar,Npar);   bvec = zeros(Npar,1); 
[~, tN] = size(Xn1); 
for i=1:Npar
    term_i = termsInUse{i};   % size = dimX x tN
    bvec(i)   = sum(sum(Xn1.*term_i) )/tN; 
    for j= 1:i
        term_j = termsInUse{j}; 
        Amat(i,j) = sum(sum(term_i.*term_j))/tN; 
        Amat(j,i) = Amat(i,j);
    end
end
cvec   = Amat\bvec;

cterms = 0; 
for i=1:Npar
    cterms = cterms + cvec(i)*termsInUse{i};  
end
res    = norm(Xn1 - cterms, 'fro')^2/(2*tN);
rankA  = rank(Amat); 
if exist('printON','var') && printON ==1; fprintf('Rank Amat = %i, Npar= %i \n',rankA, Npar); end
eigenA = eig(Amat);  fprintf('EigenA:\n'); disp(eigenA); 


cvecPinv = pinv(Amat)*bvec;  cterms =0; 
for i=1:Npar
    cterms = cterms + cvecPinv(i)*termsInUse{i};  
end
resPinv    = norm(Xn1 - cterms, 'fro')^2/(2*tN);

LSE_Abres.Amat     = Amat;      LSE_Abres.bvec    = bvec;  
LSE_Abres.cvec     = cvec;      LSE_Abres.res     = res;
LSE_Abres.cvecPinv = cvecPinv;  LSE_Abres.resPinv = resPinv; 

return

