function coef_conv_deltat(gapseq,data_fm,rmInfo,fmDataInfo,sysInfo)
%% test convergence of coefficients in the inferred scheme

% gapseq   = [1, 2, 4, 10, 20, 40, 80, 160,200];% , [320];
L_gapseq = length(gapseq);

rm_string = rmInfo.nameStrFn(gapseq(end)); 
gapStr    = strrep(rm_string,'Gap',sprintf('Gap%ito',gapseq(1)));
estimatorCov_file  = [rmInfo.rm_path, 'Convergence_',gapStr,'.mat'];  % file name
if rmInfo.lse_entrywise == 1
    estimatorCov_file  = [rmInfo.rm_path, 'Convergence_',gapStr,'_LSEentrywise.mat'];  % file name
end
rmInfo.gapStr = gapStr; 
 
if ~exist(estimatorCov_file,'file')
    estimatorSeq = cell(L_gapseq,1);
    for ll=1:L_gapseq
        obsgap        = gapseq(ll);
        rmInfo.obsgap = obsgap;
        rmInfo.deltat = fmDataInfo.dt* obsgap;   deltat = rmInfo.deltat; 
        %% downsample data and estimate the parameters
        [X, dWDeltat] = DownSample_Data(data_fm,rmInfo); % downsample data ; dWDeltat ~ sqrt(deltat)*Bn

        [cvec,res,LSE_Abres] = estimatePara_MLE(X, dWDeltat,rmInfo,sysInfo);
        estimatorSeq{ll}.cvec = cvec;
        estimatorSeq{ll}.res  = res;
        estimatorSeq{ll}.LSE_Abres = LSE_Abres;

        fprintf('Estimated para with deltat=%2.4f (Gap = %i):\n',deltat, obsgap); disp(cvec');
        fprintf('sqrt(residual)*deltat: %2.4f (vs. sigma*sqrt(dt) %2.4f\n', sqrt(res)*deltat,sqrt(deltat*2/sysInfo.beta));
    end
    save(estimatorCov_file, 'rmInfo', 'sysInfo','fmDataInfo','estimatorSeq','gapseq') 
end
load(estimatorCov_file); 

%% plot the results: convengence in deltat= Gap*dt of the coefficents cvec and covergence of res 
plot_Conv_deltat_1traj(gapseq,estimatorSeq,fmDataInfo,rmInfo); 

%% 2. compare the PDF and ACF from 1 trajectory

plotPDF_ACF = 0; 
gapStartInd =4;
if ~exist('TV_RMvsTrue_array','var') || plotPDF_ACF == 1
    % downsample and compare with the RM:

    rel_err = zeros(1,L_gapseq); rel_err_Euler = rel_err;
    TV_RMvsTrue_array  = zeros(1,L_gapseq-gapStartInd+1); TV_RM0vsTrue_array = TV_RMvsTrue_array;
    for gapInd = gapStartInd:L_gapseq
        %    gapInd        = 8; % length(gapseq)-1;          % select the gapInd here: 4-10    gapseq = [1, 2, 4, 10, 20,   40, 80, 120, 160,200];% ,
        obsgap        = gapseq(gapInd);            rmInfo.obsgap = obsgap;
        [X, dWDeltat] = DownSample_Data(data_fm,rmInfo);                  % downsample data ; dWDeltat ~ sqrt(deltat)*Bn
        cvec          = estimatorSeq{gapInd}.cvec;         % size = (Npar,K,tL,gL,MseqL)
        res           = estimatorSeq{gapInd}.res;          % size = (1,K,tL,gL,MseqL)
        % rel_err_var   = compare_1traj(X,dWDeltat,cvec,res,rmInfo,sysInfo,fmDataInfo);     % run the RM, and compare with FM and Euler
        [rel_err_var,TV_RMvsTrue, TV_RM0vsTrue,scheme_cmp]  = compare_1traj(X,dWDeltat,cvec,res,rmInfo,sysInfo,fmDataInfo);     % run the RM, and compare with FM and Euler
        rel_err(gapInd)      = rel_err_var.InferredScheme;
        rel_err_Euler(gapInd) = rel_err_var.Euler;
        %   store the total variation distances
        TV_RMvsTrue_array(gapInd-gapStartInd+1)=TV_RMvsTrue;
        TV_RM0vsTrue_array(gapInd-gapStartInd+1)=TV_RM0vsTrue;
    end
    save(estimatorCov_file,'rmInfo','rel_err_Euler','rel_err','TV_RMvsTrue_array','TV_RM0vsTrue_array','scheme_cmp','gapStartInd','-append');
end
 
plot_TVD_relErr(scheme_cmp,fmDataInfo,gapStartInd,TV_RMvsTrue_array,TV_RM0vsTrue_array,rmInfo,gapseq,rel_err,rel_err_Euler)

%{
figure;  % plot the relative error in E[X1^2];   NOT good. May be Total variation between invariant measure? 
plot(gapseq(4:10),rel_err(4:10),'-x'); hold on
plot(gapseq(4:10),rel_err_Euler(4:10),'-.*'); hold on; 
legend('Inferred scheme','Euler');
xlabel('Gap'); ylabel('Relative error E(x1^2)');
set_positionFonts; % set(gcf,'Position',  [100, 1000, 1200, 400]);
tightfig;
figname     = [rmInfo.fig_path,'fig_relErr_',gapStr,'.pdf']; 
print(figname,'-dpdf');
%}

end



 