function [ICs_Mtraj] = generateICs(sysInfo,fmDataInfo,IC_inv,Mtraj)
% generate IC samples of invariant measure from a long trajectory;
%  - produce a data file if does not exist; otherwise (if exists), load it 
%  - compute stats1traj: PDF, ACF,etc

if IC_inv ==0    % not using the invariant measure as initial distibution
     ICs_Mtraj  = fmDataInfo.getInitial(Mtraj); % generateICs(); % we need this to for adatpive learning of scheme
                                         % --- the Initial distribution can not be arbitrary.
    return; 
end

filename = [fmDataInfo.datapath,'ICdata.mat'];
if exist(filename,'file')
     load(filename,'ICs'); 
     M = length(ICs(1,:)); 
     if M>Mtraj;   ICs_Mtraj = ICs(:,1:Mtraj);       end
     return; 
end

% if the above not satisfied, generate ICs by down sampling from a long trajectory
traj_L_min = floor( Mtraj*0.1/fmDataInfo.dt); 
if fmDataInfo.tN < traj_L_min 
    fmDataInfo.tN = traj_L_min+1; 
    traj_L_min    = 0; 
end
if ~exist(fmDataInfo.datafileName,'file') || traj_L_min== 0          % if 1traj data file does not exist
    plotON =0; saveON = 0; 
    X_init =   fmDataInfo.getInitial(1);
    data_fm = generateData(sysInfo,fmDataInfo, plotON,saveON,X_init);
else
    load(fmDataInfo.datafileName,'data_fm');
end
X    = data_fm.X;
tN   = data_fm.tN;
gap  = min([ceil(tN/10000),1/data_fm.dt,ceil(tN/Mtraj)]);
ICs  = X(:,1:gap:end);    % down sample the trajectory
save(filename,'ICs');

ICs_Mtraj = ICs(:,1:Mtraj); 
return 