function estimatorSeq = inferS_1traj(xIC,rmInfo,sysInfo,fmDataInfo,gapseq,tseq,plotON,saveON)
%% inferScheme from 1traj: 
%   ===>> an efficient explicit scheme, preserving invariant measure 
%{
Algorithm: 
   + generate data from an Implicit scheme (or another that accurately approximates the invariant measure)
   + estimate the parameters; 
   + run the infered scheme 
Output 
  estimatorSeq   - a cell sequence of estimators corresponding to gapseq 
% Last updated by Fei Lu 2021/1/14;
%} 

% plotON = 0; saveON=0;  % plot the solution and the potential; and save the data_fm
rng('shuffle');   % shuffle the rand number generator: use the same for our tests
data_fm = generateData(sysInfo,fmDataInfo, plotON,saveON,xIC); % output data_fm

showRank = plotON;     
%% downsample data and estimate the parameters
gapseqL      = length(gapseq);
tseqL        = length(tseq);
estimatorSeq = cell(tseqL,gapseqL);

for tt = 1:tseqL
    data_fm1.tN   = tseq(tt); tInd = 1:tseq(tt);    % cut the data_fm up to time tseq(tt); 
    data_fm1.X    = data_fm.X(:,tInd);  
    data_fm1.dWdt = data_fm.dWdt(:,tInd);  
    for ll=1:gapseqL
        obsgap = gapseq(ll);
        rmInfo.obsgap = obsgap;
        rmInfo.deltat = fmDataInfo.dt* obsgap;
        %% downsample data and estimate the parameters
        [X, dWDeltat] = DownSample_Data(data_fm1,rmInfo); % downsample data ; dWDeltat ~ sqrt(deltat)*Bn
        
        [cvec,res,LSE_Abres] = estimatePara_MLE(X, dWDeltat,rmInfo,sysInfo,showRank);
        estimatorSeq{tt,ll}.cvec = cvec;
        estimatorSeq{tt,ll}.res  = res;
        estimatorSeq{tt,ll}.LSE_Abres = LSE_Abres;
    end
end


