function [Xtraj ,blowupFlag]= run_inferred_scheme(cvec,res,tN,x0,noiseBn,rmInfo,sysInfo)
% run the inferred scheme
%{
  Y(n+1) = Yn - deltat * coefs*termsFn + sqrt(res)*Vn   
%}

dimX = length(x0); 
if ~exist('noiseBn','var'); noiseBn=0; end 
new_noise = randn(dimX,tN);   % new noise to represent the regression residual
Xtraj     = zeros(dimX,tN+1); Xtraj(:,1)= x0;
[Npar,K]  = size(cvec); 

if K~= dimX && rmInfo.lse_entrywise ==1 
    fprintf('Error: LSE_entrywise =1, but estimator is not! \n'); return
end
    
deltat    = rmInfo.deltat; useNoise = rmInfo.useNoise; 
if K==1;         resMatdeltat = deltat*sqrt(res*2/dimX)*eye(dimX); 
elseif K==dimX;  resMatdeltat = deltat*diag(sqrt(res));
end
    
YnOFF = rmInfo.termYnOFF;    % use it in the index below

blowupFlag = 0;        
switch rmInfo.rm_type 
    case 'IT1p5'
        beta   = sysInfo.beta; gradV    = sysInfo.gradV; HessV = sysInfo.hessV; 
        LaplaceGradV = sysInfo.LaplaceGradV; 
        for t=1:tN
            if norm(x0)>1e5; fprintf('Inferred RM %s blows up at %i\n', rmInfo.rm_type, t);blowupFlag=1; return; end
            [termsVal,MatHessV] = termsIT1p5(x0,gradV,HessV,LaplaceGradV,deltat,useNoise,noiseBn,beta);
            cterms   = 0;
            for i=1:Npar
                if     K==1;    cterms = cterms + cvec(i)*termsVal{i+YnOFF};
                elseif K==dimX; cterms = cterms + diag(cvec(i,:))*termsVal{i+YnOFF};   % size(terms)= dimX x tN
                end
            end
            temp         = x0+ deltat*cterms + resMatdeltat*MatHessV*new_noise(:,t);
            Xtraj(:,t+1) = temp;
            x0           = temp;
        end
    otherwise
        termsFn  = rmInfo.termsFn; 
        for t=1:tN
            if norm(x0)>1e5; fprintf('Inferred RM %s blows up at %i\n', rmInfo.rm_type, t);blowupFlag=1; return; end
            termsVal = termsFn(x0,deltat,useNoise,noiseBn(:,t));
            cterms   = 0;
            for i=1:Npar
                if     K==1;    cterms = cterms + cvec(i)*termsVal{i+YnOFF};
                elseif K==dimX; cterms = cterms + diag(cvec(i,:))*termsVal{i+YnOFF};   % size(terms)= dimX x tN
                end
            end
            temp         = x0+ deltat*cterms + resMatdeltat*new_noise(:,t);
            Xtraj(:,t+1) = temp;
            x0           = temp;
        end
end

end