
% function rmInfo = settings_inferScheme(datapath,fmDataInfo,sysInfo)
%{ 
settings of the scheme to be inferred: 
Euler form     Y(n+1) = Yn + deltat * termsEuler(Yn,deltat) + sigma1*sqrt(deltat)*Bn 
               termsEuler(Yn,deltat) =  [c0*Yn - c1* gradV(Yn)]    
IT1p5 form     Y(n+1) = Yn + deltat * termsIT1p5(Yn,deltat) + simga1*deltat*HessV(Yn)*Wn        ******** TBD
               termsIT1p5(Yn,deltat,Bn) = [c0*Yn - c1* gradV(Yn) + c2 LgV(Yn)*deltat + ce ]   
%}

termYnOFF     = 1; % 0 to include c0*Yn; 1, remove this term

lse_entrywise = 1;  % estimate coefficients entrywisely or not


obsgap      = 100;    % observation gap for the data: RM may further downsample it; 
deltat      = fmDataInfo.dt*obsgap; 
datapath    = fmDataInfo.datapath;
dimX        = sysInfo.dimX;     % dimension of X


scheme_type  = 'RK4';  % 'Euler' 'EulerHess' 'RK4', IT1p5
useNoise     = 1;      % 1 use original noise Bn; otherwise, no 
                % Note: when the noise is not used, the rm is at
                % disadvantage when using the original noise as input,
                % because it inflates the effect of the noise by res. In
                % this case, a better test is to generate new noise for itself. 
gradV        = sysInfo.gradV; 
HessV        = sysInfo.hessV; 
LaplaceGradV = sysInfo.LaplaceGradV; 

switch   scheme_type
    case 'Euler'       % ansatz from explicit Euler
        termsFn    = @(Yn,deltat,useNoise,Bn) termsEuler(Yn,gradV,deltat,useNoise,Bn); 
    case 'EulerHess'   % ansatz from backward Euler
        hessGradFn = @(X) Hess_grad(X,HessV,gradV,deltat);   
        termsFn    = @(Yn,deltat,useNoise,Bn) termsEuler(Yn,hessGradFn,deltat,useNoise,Bn); 
    case 'RK4'       % ansatz from explicit Euler
        termsFn    = @(Yn,deltat,useNoise,Bn) termsRK4(Yn,gradV,deltat,useNoise,Bn);    
    case 'IT1p5'       % ansatz from Ito-Taylor 1.5   --- not used, because of multiple output necessary
        termsFn    = @(Yn,deltat,useNoise,Bn) termsIT1p5(Yn,gradV,HessV,LaplaceGradV,deltat,useNoise,Bn,sysInfo.beta,1);
end


fprintf('\n Ansatz scheme Type %s, useNoise = %i, obsGap= %i deltat= %2.5f YnOFF=%i\n ', scheme_type, useNoise,obsgap,deltat, termYnOFF); 
% file names
resutlsPath = [datapath,'InferScheme/'];      if ~exist(resutlsPath,'dir'); mkdir(resutlsPath); end
rm_path     = [resutlsPath, scheme_type,'/']; if ~exist(rm_path,'dir'); mkdir(rm_path); end
fig_path    = [rm_path,'figures/'];           if ~exist(fig_path,'dir'); mkdir(fig_path); end

rmInfo     = struct(...     
    'rm_type',   scheme_type,...
    'termsFn',   termsFn,...  
    'termYnOFF', termYnOFF,...
    'useNoise',  useNoise,...
    'obsgap',    obsgap,...
    'deltat',    deltat,...
    'rm_path',   rm_path, ...
    'fig_path',  fig_path,...
    'lse_entrywise', lse_entrywise,...    % estimate coefficients entrywisely
    'nameStrFn', @(obsgap) gap_string(termYnOFF,scheme_type,obsgap,useNoise)... %     'estimator_filename',  estimator_filename ...
    );
fprintf('\n Settings of inference, rmInfo: \n'); disp(rmInfo);

function stringGap = gap_string(termYnOFF,scheme_type,obsgap,useNoise)
% a function for a string output, to be used in file names. 
if termYnOFF==1
    stringGap    = [scheme_type,sprintf('Gap%i_useNoise%iYnOFF',obsgap, useNoise)];
else
    stringGap    = [scheme_type,sprintf('Gap%i_useNoise%i',obsgap, useNoise)];
end
end
