function terms = termsRK4(Yn,term_gradV,deltat,useNoise,Bn)
% the terms in EulerScheme for inference
% Euler form     Y(n+1) = Yn - deltat * termsEuler(Yn,deltat) + sigma1*sqrt(deltat)*Bn 
%                termsEuler(Yn,deltat) =  [c0.*Yn - c1* gradV(Yn)]   
% Note that Yn is a vector, so c0 is also a vector
if ~exist('useNoise','var'); useNoise = 0; end

switch useNoise 
    case 0    % RK4 output as a term
        terms    = cell(2,1);
        terms{1} = Yn;
        k1 = -term_gradV(Yn);
        k2 = -term_gradV(Yn+k1*deltat/2);
        k3 = -term_gradV(Yn+k2*deltat/2);
        k4 = -term_gradV(Yn+k3*deltat/2);
        terms{2} = (k1+2*k2+2*k3+k4)/6;
        maxVal = max([max(k1,[],'all'),max(k2,[],'all'),max(k3,[],'all'),max(k4,[],'all')] );
        if maxVal>1e5; fprintf('\n Attention: RK4 to blow up \n'); return; end
    case 1    % hybrid RK4 output as a term
        terms    = cell(3,1);
        terms{1} = Yn;
        k1 = -term_gradV(Yn)+Bn/deltat;
        k2 = -term_gradV(Yn+k1*deltat/2)+Bn/deltat;
        k3 = -term_gradV(Yn+k2*deltat/2)+Bn/deltat;
        k4 = -term_gradV(Yn+k3*deltat/2)+Bn/deltat;
        terms{2} = (k1+2*k2+2*k3+k4)/6;
        terms{3} = Bn/deltat;
        maxVal = max([max(k1,[],'all'),max(k2,[],'all'),max(k3,[],'all'),max(k4,[],'all')] ); 
         if maxVal>1e5; fprintf('\n Attention: RK4 to blow up \n'); return; end
    case 10    % estimate the coefficients in RK4
        terms    = cell(5,1);
        terms{1} = Yn;
        k1 = -term_gradV(Yn);
        k2 = -term_gradV(Yn+k1*deltat/2);
        k3 = -term_gradV(Yn+k2*deltat/2);
        k4 = -term_gradV(Yn+k3*deltat/2);
        terms{2} = k1/6;
        terms{3} = k2/3;
        terms{4} = k3/3;
        terms{5} = k4/6;
         maxVal = max([max(k1,[],'all'),max(k2,[],'all'),max(k3,[],'all'),max(k4,[],'all')] );
         if maxVal>1e5; fprintf('\n Attention: RK4 to blow up \n'); return; end
    case 11    % estimate the coefficients in RK4
        terms    = cell(5,1);
        terms{1} = Yn;
        k1 = -term_gradV(Yn)+Bn/deltat;
        k2 = -term_gradV(Yn+k1*deltat/2)+Bn/deltat;
        k3 = -term_gradV(Yn+k2*deltat/2)+Bn/deltat;
        k4 = -term_gradV(Yn+k3*deltat/2)+Bn/deltat;
        terms{2} = k1/6;
        terms{3} = k2/3;
        terms{4} = k3/3;
        terms{5} = k4/6;
        maxVal = max([max(k1,[],'all'),max(k2,[],'all'),max(k3,[],'all'),max(k4,[],'all')] ); 
         if maxVal>1e5; fprintf('\n Attention: RK4 to blow up \n'); return; end
end


end