function plot_Conv_Mtraj(Mseq,estimatorSeq,fmDataInfo,rmInfo,cvecAll,gapInd_opt)

%% plot the convergence of coeffs vs Mtraj

%% Indicator for initial condition generator
% global IC_inv
% IC_inv=0; %% for fixed initial condition
% IC_inv=1; %% for initial condition downsample (generated) from the
% invariant measure

%% ---------------------------------------------
global range 
L_gapseq = length(Mseq);
range = 1:L_gapseq;  
[L_cvec,K]  = size(estimatorSeq{1}.cvec);

normalizeBYdeltat = 0;
if K==1
    plot_LSE_nonEntrywise(Mseq,estimatorSeq,fmDataInfo,rmInfo,cvecAll,normalizeBYdeltat,gapInd_opt);
elseif K>1
    plot_LSE_Entrywise(Mseq,estimatorSeq,fmDataInfo,rmInfo,cvecAll,normalizeBYdeltat,gapInd_opt);
end
end


function plot_LSE_nonEntrywise(Mseq,estimatorSeq,fmDataInfo,rmInfo,cvecAll,normalizeBYdeltat,gapInd_opt)
global IC_inv range
L_gapseq    = length(Mseq);
[L_cvec,~]  = size(estimatorSeq{1}.cvec);
cvec_array  = zeros(L_gapseq, L_cvec);
res_array   = zeros(L_gapseq, 1);
deltat = fmDataInfo.dt*gapInd_opt;
termYnOFF   = rmInfo.termYnOFF;

%%  store the standard derivation std error of each coefficient for each given gapseq with respect to M=1000 traj
err=zeros(L_gapseq,L_cvec);k=1;
for ll=1:L_gapseq
    if normalizeBYdeltat ==0; deltat = 1; end      % remove deletat normalizing, to match the new draft
    for j=1:L_cvec
        cvec_array(ll,j)= deltat* estimatorSeq{ll}.cvec(j);
        err(ll,j)       = std((cvecAll(j,k,1,ll,:)-cvec_array(ll,j)));
    end
end

markerseq ={'-o','-x','-.>','-.+','-d-'};
figure;
lgnd = cell(L_cvec,1);
for j=1:L_cvec
    plot(log2(Mseq), cvec_array(:,j),markerseq{j},'Linewidth',1);hold on
    %% Plot errorbar for the estimators;
    %errorbar(delta_array, cvec_array(:,j),err(:,j),markerseq{j},'Linewidth',1); hold on;
    x       = log2(Mseq); y = (cvec_array(:,j));
    [rNx,cNx]=size(x);[rNy,cNy]=size(y);
    if rNx~=rNy
        x=x';
    end
    [a,~]   = polyfit(x,y,1);
    %% when termYnOFF==0, legend starts from c_0
    if termYnOFF
        lgnd{j} = [sprintf('c_%i = %2.4f ',j,a(1)),'log_2(M)',sprintf('+ %2.4f',a(2))];
    else
        lgnd{j} = [sprintf('c_%i = %2.4f ',(j-1),a(1)),'log_2(M)',sprintf('+ %2.4f',a(2))];
    end
end
legend(lgnd,'Location','best'); labelx=xlabel('$log_2(M)$'); set(labelx,'Interpreter','latex');
ylabel('Coefficients');  hold off

set_positionFonts;
%set(gcf,'Position',  [100, 1000, 1000, 400]);
tightfig;
figname = [rmInfo.fig_path, 'conv_vsM',rmInfo.gapStr,'.pdf'];

if IC_inv==0
    figname = [rmInfo.fig_path, 'conv_vsM',rmInfo.gapStr,'_FixedIC.pdf'];
end

print(figname,'-dpdf');
end


function plot_LSE_Entrywise(Mseq,estimatorSeq,fmDataInfo,rmInfo,cvecAll,normalizeBYdeltat,gapInd_opt)
global IC_inv  range
L_gapseq    = length(Mseq);
[L_cvec,K]  = size(estimatorSeq{1}.cvec);   % K = dimX; L_cvec = dim parameter

figure;
for k=1:K
    cvec_array  = zeros(L_gapseq, L_cvec);
    res_array   = zeros(L_gapseq, 1);
    deltat = fmDataInfo.dt*gapInd_opt;
    termYnOFF   = rmInfo.termYnOFF;
    
    err=zeros(L_gapseq,L_cvec); % store std for each coefficient at each given gapseq
    for ll=1:L_gapseq
        
        if normalizeBYdeltat ==0; deltat = 1; end      % remove deletat normalizing, to match the new draft
        for j=1:L_cvec
            cvec_array(ll,j)=  deltat* estimatorSeq{ll}.cvec(j,k);
            %% compute the errorbar for the estimators;
            %err(ll,j)=std((cvecAll(j,k,1,1,ll)-cvec_array(ll,j)));
        end
     end
    
    markerseq ={'-o','-x','-.>','-.+','-d-'};
    subplot(K,1,1+(k-1)*1);  lgnd = cell(L_cvec,1);
    for j=1:L_cvec
        plot(log2(Mseq), cvec_array(:,j),markerseq{j},'Linewidth',1);hold on
        
        %  if normalizeBYdeltat ==1   % remove deletat normalizing, to match the new draft
        x       = log2(Mseq); y = (cvec_array(:,j));
        [rNx,cNx]=size(x);[rNy,cNy]=size(y);
        if rNx~=rNy
            x=x';
        end
        [a,~]   = polyfit(x,y,1);
        %% when termYnOFF==0, legend starts from c_0
        if termYnOFF
            lgnd{j} = [sprintf('c_%i = %2.4f ',j,a(1)),'log_2 (M)',sprintf('+ %2.4f',a(2))];
        else
            lgnd{j} = [sprintf('c_%i = %2.4f ',(j-1),a(1)),'log_2 (M)',sprintf('+ %2.4f',a(2))];
        end
    end
    legend(lgnd,'Location','best'); 
    labelx=xlabel('$log_2(M)$'); set(labelx,'Interpreter','latex');
    ylabel('Coefficients');  
end
hold off

set_positionFonts;
%set(gcf,'Position',  [100, 1000, 1000, 100+300*K]);
tightfig;
figname = [rmInfo.fig_path, 'conv_vsM',rmInfo.gapStr,'_lseEntrywise.pdf'];
if IC_inv==0
    figname = [rmInfo.fig_path, 'conv_vsM',rmInfo.gapStr,'_FixedIC.pdf'];
end
print(figname,'-dpdf');

end
