
function plot_Conv_deltat(gapseq,estimatorSeq,fmDataInfo,rmInfo)
% plot the convergence of coefs&res in deltat= gap*dt; 

[L_cvec,K]  = size(estimatorSeq{1}.cvec);

normalizeBYdeltat = 0; 
if K==1
   plot_LSE_nonEntrywise(gapseq,estimatorSeq,fmDataInfo,rmInfo,normalizeBYdeltat);
elseif K>1
   plot_LSE_Entrywise(gapseq,estimatorSeq,fmDataInfo,rmInfo,normalizeBYdeltat);
end
end


function plot_LSE_nonEntrywise(gapseq,estimatorSeq,fmDataInfo,rmInfo,normalizeBYdeltat)
    L_gapseq    = length(gapseq);
    [L_cvec,~]  = size(estimatorSeq{1}.cvec);
    cvec_array  = zeros(L_gapseq, L_cvec);
    res_array   = zeros(L_gapseq, 1);
    delta_array = fmDataInfo.dt*gapseq;
    termYnOFF   = rmInfo.termYnOFF;
    
    for ll=1:L_gapseq
        deltat = delta_array(ll);    % obsgap = gapseq(ll); % should use deltat = obsgap *dt instead of obsgap
        if normalizeBYdeltat ==0; deltat = 1; end      % remove deletat normalizing, to match the new draft
        for j=1:L_cvec
            cvec_array(ll,j)=  deltat* estimatorSeq{ll}.cvec(j);   
        end
        res_array(ll)=  deltat* sqrt(estimatorSeq{ll}.res);  
    end
    
    markerseq ={'-o','-x','-.>','-.+','-d-'};
    figure;
    subplot(121);  lgnd = cell(L_cvec,1);
    for j=1:L_cvec
        plot(delta_array, cvec_array(:,j),markerseq{j},'Linewidth',1);hold on    %% TODO: errorbar for the estimators;
        x       = delta_array; y = cvec_array(:,j);
        [a,~]   = polyfit(x,y,1);
        %% when termYnOFF==0, legend starts from c_0
        if termYnOFF
            lgnd{j} = [sprintf('c_%i = %2.4f ',j,a(1)),'\delta',sprintf('+ %2.4f',a(2))];
        else
            lgnd{j} = [sprintf('c_%i = %2.4f ',(j-1),a(1)),'\delta',sprintf('+ %2.4f',a(2))];
        end
    end
    legend(lgnd,'Location','best'); xlabel('\delta'); ylabel('Coefficients');  hold off
    
    subplot(122);
    loglog(delta_array, res_array,'-v','Linewidth',1); hold on;        % %% TODO: errorbar for the estimators; NOTE the loglog effect --- how to do it?
    ind = 2:L_gapseq;
    x     = log10(delta_array); y = log10(res_array);
    [a,~] = polyfit(x(ind),y(ind),1);
    xlabel('\delta');
    if normalizeBYdeltat == 1    % remove deletat normalizing, to match the new draft
        ylabel('\delta\sigma_\eta ');
        lgnd  = ['log10 (\delta\sigma_\eta) ', sprintf('= %2.4f', a(1)),'log10(\delta)',sprintf('+ %2.4f', a(2))];
    else
        ylabel('\sigma_\eta ');
        lgnd  = ['log10 (\sigma_\eta) ', sprintf('= %2.4f', a(1)),'log10(\delta)',sprintf('+ %2.4f', a(2))];
    end
    
    legend(lgnd,'Location','best');
    
    set_positionFonts;
    set(gcf,'Position',  [100, 1000, 1000, 400]);tightfig;
    figname = [rmInfo.fig_path, 'conv_',rmInfo.gapStr,'.pdf'];
    print(figname,'-dpdf');
end


function plot_LSE_Entrywise(gapseq,estimatorSeq,fmDataInfo,rmInfo,normalizeBYdeltat)
L_gapseq    = length(gapseq);
[L_cvec,K]  = size(estimatorSeq{1}.cvec);
 
figure;
for k=1:K
    cvec_array  = zeros(L_gapseq, L_cvec);
    res_array   = zeros(L_gapseq, 1);
    delta_array = fmDataInfo.dt*gapseq;
    termYnOFF   = rmInfo.termYnOFF;
    
    for ll=1:L_gapseq
        deltat = delta_array(ll);    % obsgap = gapseq(ll); % should use deltat = obsgap *dt instead of obsgap
        if normalizeBYdeltat ==0; deltat = 1; end      % remove deletat normalizing, to match the new draft
        for j=1:L_cvec
            cvec_array(ll,j)=  deltat* estimatorSeq{ll}.cvec(j);
        end
        res_array(ll)=  deltat* sqrt(estimatorSeq{ll}.res);
    end
        
    markerseq ={'-o','-x','-.>','-.+','-d-'};
    subplot(K,2,1+(k-1)*2);  lgnd = cell(L_cvec,1);
    for j=1:L_cvec
        plot(delta_array, cvec_array(:,j),markerseq{j},'Linewidth',1);hold on    %% TODO: errorbar for the estimators;
        
      %  if normalizeBYdeltat ==1   % remove deletat normalizing, to match the new draft
        x       = delta_array; y = cvec_array(:,j);
        [a,~]   = polyfit(x,y,1);
        %% when termYnOFF==0, legend starts from c_0
        if termYnOFF
            lgnd{j} = [sprintf('c_%i = %2.4f ',j,a(1)),'\delta',sprintf('+ %2.4f',a(2))];
        else
            lgnd{j} = [sprintf('c_%i = %2.4f ',(j-1),a(1)),'\delta',sprintf('+ %2.4f',a(2))];
        end
    end
    legend(lgnd,'Location','best'); xlabel('\delta'); ylabel(sprintf('Coef. of x%i',k));  hold off
    
    subplot(K,2,2+(k-1)*2);
    loglog(delta_array, res_array,'-v','Linewidth',1); hold on;        % %% TODO: errorbar for the estimators; NOTE the loglog effect --- how to do it?
    ind = 2:L_gapseq;
    x     = log10(delta_array); y = log10(res_array);
    [a,~] = polyfit(x(ind),y(ind),1);
    xlabel('\delta');
    if normalizeBYdeltat == 1    % remove deletat normalizing, to match the new draft
        ylabel(['\delta\sigma_\eta ',sprintf('x%i',k)]);
        lgnd  = ['log10 (\delta\sigma_\eta) ', sprintf('= %2.4f', a(1)),'log10(\delta)',sprintf('+ %2.4f', a(2))];
    else
        ylabel(['\sigma_\eta ',sprintf('x%i',k)]);
        lgnd  = ['log10 (\sigma_\eta) ', sprintf('= %2.4f', a(1)),'log10(\delta)',sprintf('+ %2.4f', a(2))];
    end
    legend(lgnd,'Location','best');
    hold on; 
end
hold off
set_positionFonts;
set(gcf,'Position',  [100, 1000, 1000, 100+300*K]);tightfig;
figname = [rmInfo.fig_path, 'conv_',rmInfo.gapStr,'_lseEntrywise.pdf'];
print(figname,'-dpdf');

end