%%  run SDE: 
clear all; close all; clc

%% set paths
addpath_inferScheme;   % adding path, and save data to local DIR

%% demonstrate a single trajectory
scheme =  'SSBE';  


% 1. run 1D double-well 
potential_type = 'LangLocalGradSys'; 
[sysInfo,fmDataInfo]  = potential1D(potential_type,scheme);  
fmDataInfo.tN = 1e4; fmDataInfo.dt = 1e-2;   % large timestep than default for demonstration. NOT accurate. 
[data_fm,~]  = SplitStepBackwardEuler_v2(sysInfo,fmDataInfo);  
% data_fm: a structrue with true trajectory, 
%    'dWdt',     dWdt, ...           % increment of stochastic force *dt; 
%     'tN',       tN, ...            % length of trajectory
%    'dt',       dt, ...             % time-step size 
%    'Eig_Newton',  Eig_Newton ...   % eigenvalue in Newton iteration 
%    );
 tt = data_fm.dt*(1:data_fm.tN);
figure; 
subplot(211); 
   plot(tt,data_fm.X(1,1:end-1)); xlabel('Time t'); ylabel('X(t)');
   title('A trajectory of X(t): Double-well potential 1D'); 
subplot(212); 
   plot(tt,data_fm.dWdt(1,1:end)); xlabel('Time t'); ylabel('dWdt');
   title('Increments of the stochastic force *dt'); 



%2. run a 2D gradient system with uncoupled potential 
potential_type = 'GradDiss'; 
[sysInfo,fmDataInfo]  = GradDiss_settings(potential_type,scheme); 
fmDataInfo.tN = 1e4; fmDataInfo.dt = 1e-2;      % large timestep than default for demonstration. NOT accurate.
[data_fm,~]  = SplitStepBackwardEuler_v2(sysInfo,fmDataInfo);  
 tt = data_fm.dt*(1:data_fm.tN);
figure; subplot(211);
   plot(tt,data_fm.X(1,1:end-1)); xlabel('Time t'); ylabel('X(t)');
   title('Trajectory of X1(t): 2D Gradient system V(x,y) = V1(x) + V2(y)'); 
subplot(212); 
   plot(tt,data_fm.X(2,1:end-1)); xlabel('Time t'); ylabel('dWdt');
   title('Trajectory of X2(t)'); 


% 3. stochastic Lorenz 63: 
potential_type = 'Lorenz'; 
[sysInfo,fmDataInfo]  = Lorenz_settings(potential_type,scheme);
fmDataInfo.tN = 5*1e4; fmDataInfo.dt = 1e-3;      % large timestep than default for demonstration. NOT accurate.
[data_fm,~]  = SplitStepBackwardEuler_v2(sysInfo,fmDataInfo);  
 tt = data_fm.dt*(1:data_fm.tN);
figure; subplot(211);
   plot(tt,data_fm.X(1,1:end-1)); xlabel('Time t'); ylabel('X(t)');
   title('Trajectory of X1(t): stochastic Lorenz63'); 
subplot(212); 
   plot(tt,data_fm.X(2,1:end-1)); xlabel('Time t'); ylabel('dWdt');
   title('Trajectory of X2(t)'); 


