%% inferScheme -- infer a numerical scheme with a larger time step size
%  + as fast as an explicit scheme, also get invariant measure as an Implicit scheme
%{
Algorithm: 
   + generate data from an Implicit scheme (or another that accurately approximates the invariant measure)
   + estimate the parameters; 
   + run the infered scheme  (You can change ansatz settings inside inferS_1traj_demo.m ) 
NOTE: test different time step-sizes-- how the coeffecients depend on it? 
% Copyright: Xingjie Li and Fei Lu;    Last updated: 2021-4-28  startetd 2020-12-23  
%}
%{
About how to run the code:
 - run this code: select a setting in model_settings_all.m
 - test new settings: go into inferS_1traj_demo.m:  change ansatz settings in settings_inferScheme.m and run inferS_1traj_demo.m)
 - To see results: All resutls are saved in the SAVDDIR

Data File structure: results are saved for Each sample: 
  - Example name 
     - setting_tN_solver
        + Data*: trajector of full model
        - inferScheme 
           - RM_type
             + Estimator* (end with M20 if Mtraj>1; otherwise, end with YnOFF if YnOFF-- otherwise, end with useNoise
                          when M>1: all estimators are saved (gapseq,tseq,Mseq);
                          when M=1: save estimator for only 1 gap; estimators of gapseq is saved in Convergence 
             + Convergence*: estimator for gapseq when M=1; 
             + figData*:     PDF and ACF data for plots; 
%}

clear all; close all; clc

%% set paths
restoredefaultpath; 
addpath_inferScheme;   % adding path, and save data to local DIR

%% Start from here: Set Parameters of full system and reduced models (RM) 
[sysInfo,fmDataInfo] = model_settings_all(SAVE_DIR,myslash);   % change settings of full model and reduced models 
fprintf('sysInfo:\n'); disp(sysInfo);fprintf('fmDataInfo:\n'); disp(fmDataInfo); 

settings_inferScheme;  % settings should have been decided for M_traj: time consuming to compute 
if ~isfield(rmInfo,'lse_entrywise'); lse_entrywise = 0; rmInfo.lse_entrywise = lse_entrywise; end   % estimate LSE entrywise

Mtraj  = 1;     % number of trajectories for inference; 
gapseq = [1, 2, 4, 10, 20, 40, 80, 120, 160,200];% , [320];
if isfield(fmDataInfo,'gapseq'); gapseq= fmDataInfo.gapseq; end
rmInfo.nameStrFn = @(n)[rmInfo.nameStrFn(n), sprintf('_M%i',Mtraj)];   % add _M# to all the file names   

if Mtraj==1
    %% demonstrate Model reduciton using a single trajectory
    plotON = 0; saveON =1;  % plot the solution and the potential; and save the data_fm    
    rng(1);                 % shuffle the rand number generator: use the same for our tests
    data_fm = generateData(sysInfo,fmDataInfo, plotON,saveON); % output data_fm
    % infer the reduced model: settings for inference inside, so as to test different settings 
    inferS_1traj_demo;      % estimates a scheme for the closure; run it and compare its statistics.      
else
    %% inference from many trajectories
    IC_inv = 1; % initial condition from invariant measure; 
    ICs    = generateICs(sysInfo,fmDataInfo,IC_inv,Mtraj); % generate initial conditions by downsampling a long traj; save PDF,ACF,  
  
    inferScheme_Mtraj; % (ICs,sysInfo,fmDataInfo,rmInfo,Mtraj,gapseq); 
                       % results presentation inside
end


