function [data_orig,X,blowupFlag] = hybridRK4(sysInfo,fmDataInfo,X_init,dWdt)
% solve the system by Euler-Maruyama integrator: 
%{ 
        dX = f(X)dt + b(X)dW 
 With  
     X(:,n+1) = X(:,n)+ f(Xn)*dt + b*sqrt(dt) * dW
 Output
    data_orig  
%}
dimX        = sysInfo.dimX; 
drift_neg   = sysInfo.gradV;   
diffMat     = sysInfo.diffusion; 

dt    =   fmDataInfo.dt;  
tN    =   fmDataInfo.tN;
if nargin <3
    X_init =   fmDataInfo.getInitial(1);
end
if ~exist('dWdt','var')  
    dWdt   =   sqrt(dt)*randn(dimX,tN);  % i.i.d increments of Bm for the integrator.
end

X      = zeros(dimX,tN+1);
X(:,1) = X_init;
x0     = X_init;
blowupFlag = 0; i = 1; 


RK4_EM =  1; % use RK4+EM or use RKE with stochastic force as constant in each time step
if RK4_EM == 0
    dWdt_scaled = diffMat* dWdt/dt;   % note the scaling /dt here  !!!
    while i <tN
        if norm(x0)>1e5; fprintf('Hybrid RK4 blows up at time= %i\n', i); blowupFlag=1;  end
        Bn = dWdt_scaled(:,i);
        k1 = drift_neg(x0)+Bn;
        k2 = drift_neg(x0+k1*dt/2)+Bn/dt;
        k3 = drift_neg(x0+k2*dt/2)+Bn/dt;
        k4 = drift_neg(x0+k3*dt/2)+Bn/dt;
        x1 = x0- dt*(k1+2*k2+2*k3+k4)/6;
        X(:,i+1) = x1; x0 = x1;
        i  = i+1;
    end
else
    dWdt_scaled = diffMat* dWdt;
    while i <tN
        if norm(x0)>1e5; fprintf('Hybrid RK4 blows up at time= %i\n', i); blowupFlag=1;  end
        k1 = drift_neg(x0);
        k2 = drift_neg(x0+k1*dt/2);
        k3 = drift_neg(x0+k2*dt/2);
        k4 = drift_neg(x0+k3*dt/2);
        x1 = x0- dt*(k1+2*k2+2*k3+k4)/6 + dWdt_scaled(:,i);
        X(:,i+1) = x1; x0 = x1;
        i  = i+1;
    end
end

data_orig = struct( ...
    'X',        X,...
    'dWdt',     dWdt, ...
    'tN',       tN, ...
    'dt',       dt ...
);
end

